@@ -100,11 +100,6 @@
 // the build system #defining vxworks.
 #if defined(__vxworks) || defined(vxworks)
 #   define NV_VXWORKS
-#   if (CPU==PENTIUM)
-#      define NV_VXWORKS_PENTIUM
-#   elif (CPU==PPC604)
-#      define NV_VXWORKS_PPC604
-#   endif
 #endif
 
 // ***** Integrity OS
@@ -121,6 +116,7 @@
     // _M_IX86 for windows, __i386__ for Linux (or any x86 using gcc)
     // __i386 for Studio compiler on Solaris x86
 #   define NVCPU_X86               // any IA32 machine (not x86-64)
+#   define NVCPU_MIN_PAGE_SHIFT 12
 #endif
 
 #if defined(_WIN32) && defined(_M_IA64)
@@ -146,12 +142,9 @@
 #    define NVCPU_X86_64           // any x86-64 for any opsys
 #endif
 
-#if defined(__arm__) && !defined(NVCPU_ARM)
+#if defined(__arm__) || defined(_M_ARM)
 #   define NVCPU_ARM
-#endif
-
-#if defined(__XSCALE__)
-#define NVCPU_XSCALE               // for XSCALE ARM processors
+#   define NVCPU_MIN_PAGE_SHIFT 12
 #endif
 
 #if defined(__SH4__)
@@ -176,6 +169,11 @@
 // compiler for certain files in a Linux build.
 
 
+// The minimum page size can be determined from the minimum page shift
+#if defined(NVCPU_MIN_PAGE_SHIFT)
+#define NVCPU_MIN_PAGE_SIZE (1 << NVCPU_MIN_PAGE_SHIFT)
+#endif
+
 #if defined(NVCPU_IA64) || defined(NVCPU_X86_64) || defined(NV_MACINTOSH_64)
 #   define NV_64_BITS          // all architectures where pointers are 64 bits
 #else
@@ -195,6 +193,115 @@
 #define NV_VERIF_FEATURES
 #endif
 
+
+/*
+ * New, safer family of #define's -- these ones use 0 vs. 1 rather than
+ * defined/!defined.  This is advantageous because if you make a typo, say:
+ *
+ *   #if NVCPU_IS_BIG_ENDAIN    // Oops!  Endian is misspelled
+ *
+ * ...some compilers can give you a warning telling you that you screwed up.
+ * The compiler can also give you a warning if you forget to #include
+ * "cpuopsys.h" in your code before the point where you try to use these
+ * conditionals.
+ *
+ * Also, the names have been prefixed in more cases with "CPU" or "OS" for
+ * increased clarity.  You can tell the names apart from the old ones because
+ * they all use "_IS_" in the name.
+ *
+ * Finally, these can be used in "if" statements and not just in #if's.  For
+ * example:
+ *
+ *   if (NVCPU_IS_BIG_ENDIAN) x = Swap32(x);
+ *
+ * Maybe some day in the far-off future these can replace the old #define's.
+ */
+#if defined(NV_WINDOWS)
+#define NVOS_IS_WINDOWS 1
+#else
+#define NVOS_IS_WINDOWS 0
+#endif
+#if defined(NV_WINDOWS_CE)
+#define NVOS_IS_WINDOWS_CE 1
+#else
+#define NVOS_IS_WINDOWS_CE 0
+#endif
+#if defined(NV_LINUX)
+#define NVOS_IS_LINUX 1
+#else
+#define NVOS_IS_LINUX 0
+#endif
+#if defined(NV_UNIX)
+#define NVOS_IS_UNIX 1
+#else
+#define NVOS_IS_UNIX 0
+#endif
+#if defined(NV_QNX)
+#define NVOS_IS_QNX 1
+#else
+#define NVOS_IS_QNX 0
+#endif
+#if defined(NV_MACINTOSH)
+#define NVOS_IS_MACINTOSH 1
+#else
+#define NVOS_IS_MACINTOSH 0
+#endif
+#if defined(NV_VXWORKS)
+#define NVOS_IS_VXWORKS 1
+#else
+#define NVOS_IS_VXWORKS 0
+#endif
+#if defined(NV_INTEGRITY)
+#define NVOS_IS_INTEGRITY 1
+#else
+#define NVOS_IS_INTEGRITY 0
+#endif
+#if defined(NVCPU_X86)
+#define NVCPU_IS_X86 1
+#else
+#define NVCPU_IS_X86 0
+#endif
+#if defined(NVCPU_IA64)
+#define NVCPU_IS_IA64 1
+#else
+#define NVCPU_IS_IA64 0
+#endif
+#if defined(NVCPU_X86_64)
+#define NVCPU_IS_X86_64 1
+#else
+#define NVCPU_IS_X86_64 0
+#endif
+#if defined(NVCPU_PPC)
+#define NVCPU_IS_PPC 1
+#else
+#define NVCPU_IS_PPC 0
+#endif
+#if defined(NVCPU_ARM)
+#define NVCPU_IS_ARM 1
+#else
+#define NVCPU_IS_ARM 0
+#endif
+#if defined(NVCPU_SH4)
+#define NVCPU_IS_SH4 1
+#else
+#define NVCPU_IS_SH4 0
+#endif
+#if defined(NVCPU_XTENSA)
+#define NVCPU_IS_XTENSA 1
+#else
+#define NVCPU_IS_XTENSA 0
+#endif
+#if defined(NV_BIG_ENDIAN)
+#define NVCPU_IS_BIG_ENDIAN 1
+#else
+#define NVCPU_IS_BIG_ENDIAN 0
+#endif
+#if defined(NV_64_BITS)
+#define NVCPU_IS_64_BITS 1
+#else
+#define NVCPU_IS_64_BITS 0
+#endif
+
 /*****************************************************************************/
 
 #endif /* __cpuopsys_h_ */
