@@ -74,6 +74,7 @@
 #include <vm/vm_object.h>
 #include <vm/pmap.h>
 #include <vm/vm_map.h>
+#include <vm/uma.h>
 
 #include <pci/agpvar.h>
 #include <sys/agpio.h>
@@ -197,6 +198,12 @@ struct nvidia_softc {
     void *irq_ih;
     int   irq_rid;
 
+    nv_stack_t *attach_sp;
+    nv_stack_t *api_sp;
+    nv_stack_t *pci_cfgchk_sp;
+    nv_stack_t *isr_sp;
+    nv_stack_t *timer_sp;
+
     struct resource *iop;
     int iop_rid;
 
@@ -267,6 +274,23 @@ extern nv_parm_t nv_parms[];
 #define __NV_IOC_TYPE(_cmd) (((_cmd) >> 8) & 0xff)
 #define __NV_IOC_NR(_cmd)   (((_cmd) >> 0) & 0xff)
 
+extern uma_zone_t nvidia_stack_t_zone;
+
+#define NV_UMA_ZONE_ALLOC_STACK(ptr)                             \
+    {                                                            \
+        (ptr) = uma_zalloc(nvidia_stack_t_zone, M_WAITOK);       \
+        if ((ptr) != NULL)                                       \
+        {                                                        \
+            (ptr)->size = sizeof((ptr)->stack);                  \
+            (ptr)->top = (ptr)->stack + (ptr)->size;             \
+        }                                                        \
+    }
+
+#define NV_UMA_ZONE_FREE_STACK(ptr)                              \
+    {                                                            \
+        uma_zfree(nvidia_stack_t_zone, (ptr));                   \
+    }
+
 /* nvidia_dev.c */
 int    nvidia_dev_attach     (struct nvidia_softc *);
 int    nvidia_dev_detach     (struct nvidia_softc *);
@@ -276,18 +300,19 @@ int    nvidia_ctl_attach     (void);
 int    nvidia_ctl_detach     (void);
 
 /* nvidia_pci.c */
-void   nvidia_pci_check_config_space  (device_t dev, BOOL);
+void   nvidia_pci_check_config_space  (nv_stack_t *, device_t dev, BOOL, BOOL, BOOL);
 int    nvidia_pci_setup_intr          (device_t dev);
 int    nvidia_pci_teardown_intr       (device_t dev);
 U008   nvidia_pci_find_capability     (device_t dev, U008);
 
-#define NV_PCI_CHECK_CONFIG_SPACE(_nv, do_the_bars) {               \
-       struct nvidia_softc *__sc;                                   \
-       if (((_nv)->flags & NV_FLAG_CONTROL) == 0) {                 \
-           __sc = (_nv)->os_state;                                  \
-           nvidia_pci_check_config_space(__sc->dev, do_the_bars);   \
-       }                                                            \
-  }
+#define NV_PCI_CHECK_CONFIG_SPACE(_sp,_nv,_cb,_as,_mb)                   \
+    {                                                                    \
+        struct nvidia_softc *__sc;                                       \
+        if (((_nv)->flags & NV_FLAG_CONTROL) == 0) {                     \
+            __sc = (_nv)->os_state;                                      \
+            nvidia_pci_check_config_space(_sp,__sc->dev,_cb,_as,_mb);    \
+        }                                                                \
+    }
 
 /* nvidia_subr.c */
 int    nvidia_attach         (device_t);
@@ -350,9 +375,8 @@ void   nvidia_linux_exit     (void);
 void   nvidia_update_registry (char *);
 
 /* ioctl helpers */
-int    nvidia_post_vbios      (struct nv_ioctl_post_vbios *);
-int    nvidia_get_card_info   (struct nv_ioctl_card_info *);
-int    nvidia_handle_ioctl    (struct cdev *, u_long, caddr_t, int, d_thread_t *);
+int    nvidia_get_card_info  (struct nv_ioctl_card_info *);
+int    nvidia_handle_ioctl   (struct cdev *, u_long, caddr_t, int, d_thread_t *);
 
 
 /* device helpers */
