@@ -77,12 +77,18 @@ int nvidia_ctl_ioctl(
 {
     int status = 0;
     nv_state_t *nv = &nvidia_ctl_state;
+    struct nvidia_softc *sc;
+    nv_stack_t *sp;
+
+    sc = nv->os_state;
 
     if (__NV_IOC_TYPE(cmd) != NV_IOCTL_MAGIC)
         return ENOTTY;
 
     nv_lock_api(nv);
 
+    sp = sc->api_sp;
+
     switch (__NV_IOC_NR(cmd)) {
         case NV_ESC_CARD_INFO:
             do {
@@ -91,17 +97,10 @@ int nvidia_ctl_ioctl(
             } while (0);
             break;
 
-        case NV_ESC_POST_VBIOS:
-            do {
-                struct nv_ioctl_post_vbios *pc = (void *) data;
-                status = nvidia_post_vbios(pc);
-            } while (0);
-            break;
-
         case NV_ESC_CHECK_VERSION_STR:
             do {
                 nv_ioctl_rm_api_version_t *params = (void *) data;
-                status = (rm_perform_version_check(NULL, params) == RM_OK) ? 0 : -EINVAL;
+                status = (rm_perform_version_check(sp, params) == RM_OK) ? 0 : -EINVAL;
             } while (0);
             break;
 
@@ -110,32 +109,10 @@ int nvidia_ctl_ioctl(
     }
 
     nv_unlock_api(nv);
+
     return status;
 }
 
-
-/*
- * The X driver module adds the ctrl device fd to the XFree86 select list
- * and receives notification if the select status changes. We make use of
- * this mechanism to signal hotkey events; this function is called by the
- * core resource manager whenever a legacy hotkey event is detected.
- */
-
-void nv_set_hotkey_occurred_flag(void)
-{
-    nv_state_t *nv;
-    struct nvidia_softc *sc;
-
-    nv = &nvidia_ctl_state;
-    sc = nv->os_state;
-
-    nv_lock_rm(nv);
-    nv->flags |= NV_FLAG_HOTKEY_OCCURRED;
-    nv_unlock_rm(nv);
-
-    selwakeup(&sc->rsel);
-}
-
 int nvidia_ctl_poll(
     struct cdev *dev,
     int events,
@@ -144,20 +121,24 @@ int nvidia_ctl_poll(
 {
     nv_state_t *nv;
     struct nvidia_softc *sc;
+    struct nvidia_event *et;
 
     nv = &nvidia_ctl_state;
     sc = nv->os_state;
 
     nv_lock_rm(nv);
 
-    if ((nv->flags & NV_FLAG_HOTKEY_OCCURRED) == 0) {
+    STAILQ_FOREACH(et, &sc->event_queue, queue) {
+        if (et->event.file == __TD_FDT(td))
+            break;
+    }
+
+    if (et == NULL) {
         nv_unlock_rm(nv);
         selrecord(td, &sc->rsel);
     } else {
-        nv->flags &= ~NV_FLAG_HOTKEY_OCCURRED;
         nv_unlock_rm(nv);
-
-        return (events & (POLLIN | POLLRDNORM)); /* ? */
+        return (events & (POLLIN | POLLPRI | POLLRDNORM));
     }
 
     return 0;
