@@ -25,8 +25,7 @@ MALLOC_DEFINE(M_NVIDIA, "nvidia", "NVIDIA memory allocations");
 
 #define MAX_ERROR_STRING    256
 
-
-RM_STATUS os_alloc_contig_pages(
+RM_STATUS NV_API_CALL os_alloc_contig_pages(
     void **address,
     U032 size
 )
@@ -35,7 +34,7 @@ RM_STATUS os_alloc_contig_pages(
     return *address ? RM_OK : RM_ERROR;
 }
 
-void os_free_contig_pages(
+void NV_API_CALL os_free_contig_pages(
     void *address,
     U032 size
 )
@@ -49,7 +48,7 @@ void os_free_contig_pages(
  * not be allowed to sleep when memory is low.
  */
 
-RM_STATUS os_alloc_mem(
+RM_STATUS NV_API_CALL os_alloc_mem(
     void **address,
     U032 size
 )
@@ -59,7 +58,7 @@ RM_STATUS os_alloc_mem(
     return *address ? RM_OK : RM_ERROR;
 }
 
-void os_free_mem(void *address)
+void NV_API_CALL os_free_mem(void *address)
 {
     free(address, M_NVIDIA);
 }
@@ -69,7 +68,7 @@ void os_free_mem(void *address)
 #define NV_USECS_PER_TICK       (1000000 / hz)
 #define NV_USECS_TO_TICKS(us)   ((us) * hz / 1000000)
 
-RM_STATUS os_delay(U032 MilliSeconds)
+RM_STATUS NV_API_CALL os_delay(U032 MilliSeconds)
 {
     unsigned long MicroSeconds;
     unsigned long ticks;
@@ -112,7 +111,7 @@ RM_STATUS os_delay(U032 MilliSeconds)
     return RM_OK;
 }
 
-RM_STATUS os_delay_us(U032 MicroSeconds)
+RM_STATUS NV_API_CALL os_delay_us(U032 MicroSeconds)
 {
     if (__NV_ITHREAD() && (MicroSeconds > NV_MAX_ISR_DELAY_US))
         return RM_ERROR;
@@ -120,18 +119,17 @@ RM_STATUS os_delay_us(U032 MicroSeconds)
     return RM_OK;
 }
 
-U032 os_get_cpu_frequency(void)
+U032 NV_API_CALL os_get_cpu_frequency(void)
 {
-    /* XXX Fix me! (x86 only) */
     return ((tsc_freq + 4999) / 1000000);
 }
 
-U032 os_get_current_process(void)
+U032 NV_API_CALL os_get_current_process(void)
 {
     return curproc->p_pid;
 }
 
-RM_STATUS os_kill_process(
+RM_STATUS NV_API_CALL os_kill_process(
     U032 pid,
     U032 sig
 )
@@ -150,7 +148,7 @@ RM_STATUS os_kill_process(
     return RM_OK;
 }
 
-RM_STATUS os_get_current_time(
+RM_STATUS NV_API_CALL os_get_current_time(
     U032 *sec,
     U032 *usec
 )
@@ -165,12 +163,12 @@ RM_STATUS os_get_current_time(
     return RM_OK;
 }
 
-BOOL os_is_administrator(PHWINFO pDev)
+BOOL NV_API_CALL os_is_administrator(PHWINFO pDev)
 {
     return suser(CURTHREAD) ? FALSE : TRUE;
 }
 
-U008 os_io_read_byte(
+U008 NV_API_CALL os_io_read_byte(
     PHWINFO pDev,
     U032 address
 )
@@ -179,7 +177,7 @@ U008 os_io_read_byte(
     return inb(address);
 }
 
-void os_io_write_byte(
+void NV_API_CALL os_io_write_byte(
     PHWINFO pDev,
     U032 address,
     U008 value
@@ -189,7 +187,7 @@ void os_io_write_byte(
     outb(address, value);
 }
 
-U016 os_io_read_word(
+U016 NV_API_CALL os_io_read_word(
     PHWINFO pDev,
     U032 address
 )
@@ -198,7 +196,7 @@ U016 os_io_read_word(
     return inw(address);
 }
 
-void os_io_write_word(
+void NV_API_CALL os_io_write_word(
     PHWINFO pDev,
     U032 address,
     U016 value
@@ -208,7 +206,7 @@ void os_io_write_word(
     return outw(address, value);
 }
 
-U032 os_io_read_dword(
+U032 NV_API_CALL os_io_read_dword(
     PHWINFO pDev,
     U032 address
 )
@@ -217,7 +215,7 @@ U032 os_io_read_dword(
     return inl(address);
 }
 
-void os_io_write_dword(
+void NV_API_CALL os_io_write_dword(
     PHWINFO pDev,
     U032 address,
     U032 value
@@ -238,7 +236,7 @@ void os_io_write_dword(
  * cached direct mapping.
  */
 
-void* os_map_kernel_space(
+void* NV_API_CALL os_map_kernel_space(
     NvU64 start,
     NvU64 size,
     U032 mode
@@ -273,13 +271,13 @@ void* os_map_kernel_space(
     return vm;
 }
 
-void os_unmap_kernel_space(
+void NV_API_CALL os_unmap_kernel_space(
     void *address,
     NvU64 size
 )
 {
     vm_offset_t tva, va;
-    
+
     size = NV_ALIGN_UP(size, PAGE_SIZE);
 
     va = (vm_offset_t) address;
@@ -291,7 +289,7 @@ void os_unmap_kernel_space(
     }
 }
 
-void* os_map_kernel_space_high(
+void* NV_API_CALL os_map_kernel_space_high(
     U032 pfn,
     U032 size
 )
@@ -304,7 +302,7 @@ void* os_map_kernel_space_high(
     return NULL;
 }
 
-void os_unmap_kernel_space_high(
+void NV_API_CALL os_unmap_kernel_space_high(
     void *addr,
     U032 pfn,
     U032 size
@@ -313,7 +311,7 @@ void os_unmap_kernel_space_high(
     os_unmap_kernel_space(addr, size);
 }
 
-RM_STATUS os_set_mem_range(
+RM_STATUS NV_API_CALL os_set_mem_range(
     U032 start,
     U032 size,
     U032 mode
@@ -335,7 +333,7 @@ RM_STATUS os_set_mem_range(
     return RM_OK;
 }
 
-RM_STATUS os_unset_mem_range(
+RM_STATUS NV_API_CALL os_unset_mem_range(
     U032 start,
     U032 size
 )
@@ -362,23 +360,30 @@ RM_STATUS os_unset_mem_range(
 
 U032 cur_debuglevel = 0xffffffff;
 
-void os_dbg_init(void)
+void NV_API_CALL os_dbg_init(void)
 {
     U032 new_debuglevel;
+    nv_stack_t *sp;
 
-    if (rm_read_registry_dword(NULL, NULL, "NVreg", "ResmanDebugLevel",
+    NV_UMA_ZONE_ALLOC_STACK(sp);
+    if (sp == NULL)
+        return;
+
+    if (rm_read_registry_dword(sp, NULL, "NVreg", "ResmanDebugLevel",
             &new_debuglevel) == RM_OK) {
         if (new_debuglevel != 0xffffffff)
             cur_debuglevel = new_debuglevel;
     }
+
+    NV_UMA_ZONE_FREE_STACK(sp);
 }
 
-void os_dbg_set_level(U032 new_debuglevel)
+void NV_API_CALL os_dbg_set_level(U032 new_debuglevel)
 {
     cur_debuglevel = new_debuglevel;
 }
 
-void NV_STACKWATCH_CALLBACK os_dbg_breakpoint(void)
+void NV_API_CALL NV_STACKWATCH_CALLBACK os_dbg_breakpoint(void)
 {
 #ifdef DEBUG
     /* FreeBSD 5.x kdb */
@@ -386,7 +391,7 @@ void NV_STACKWATCH_CALLBACK os_dbg_breakpoint(void)
 #endif
 }
 
-void out_string(const char *message)
+void NV_API_CALL out_string(const char *message)
 {
 #if defined(DEBUG) || defined(QA_BUILD)
     printf("%s", message);
@@ -395,7 +400,7 @@ void out_string(const char *message)
 
 static char nv_error_string[MAX_ERROR_STRING];
 
-int nv_printf(
+int NV_API_CALL nv_printf(
     U032 debuglevel,
     const char *format,
     ...
@@ -415,7 +420,7 @@ int nv_printf(
     return chars_written;
 }
 
-int nv_snprintf(
+int NV_API_CALL nv_snprintf(
     char *buf,
     unsigned int size,
     const char *fmt,
@@ -432,7 +437,7 @@ int nv_snprintf(
     return chars_written;
 }
 
-void nv_os_log(
+void NV_API_CALL nv_os_log(
     int loglevel,
     const char *fmt,
     void *ap
@@ -445,7 +450,7 @@ void nv_os_log(
     printf("%s", nv_error_string);
 }
 
-S032 os_mem_cmp(
+S032 NV_API_CALL os_mem_cmp(
     const U008 *buf0,
     const U008 *buf1,
     U032 length
@@ -454,7 +459,7 @@ S032 os_mem_cmp(
     return memcmp(buf0, buf1, length);
 }
 
-U008* os_mem_copy(
+U008* NV_API_CALL os_mem_copy(
     U008 *dst,
     const U008 *src,
     U032 length
@@ -469,7 +474,7 @@ U008* os_mem_copy(
 #endif
 }
 
-RM_STATUS os_memcpy_from_user(
+RM_STATUS NV_API_CALL os_memcpy_from_user(
     void *dst,
     const void *src,
     U032 length
@@ -481,7 +486,7 @@ RM_STATUS os_memcpy_from_user(
     return os_mem_copy(dst, src, length) ? RM_ERR_INVALID_POINTER : RM_OK;
 }
 
-RM_STATUS os_memcpy_to_user(
+RM_STATUS NV_API_CALL os_memcpy_to_user(
     void *dst,
     const void *src,
     U032 length
@@ -493,7 +498,7 @@ RM_STATUS os_memcpy_to_user(
     return os_mem_copy(dst, src, length) ? RM_ERR_INVALID_POINTER : RM_OK;
 }
 
-void* os_mem_set(
+void* NV_API_CALL os_mem_set(
     void *b,
     U008 c,
     U032 length
@@ -502,7 +507,7 @@ void* os_mem_set(
     return memset(b, c, length);
 }
 
-S032 os_string_compare(
+S032 NV_API_CALL os_string_compare(
     const U008 *s1,
     const U008 *s2
 )
@@ -510,7 +515,7 @@ S032 os_string_compare(
     return strcmp(s1, s2);
 }
 
-U008* os_string_copy(
+U008* NV_API_CALL os_string_copy(
     U008 *dst,
     const U008 *src
 )
@@ -518,12 +523,12 @@ U008* os_string_copy(
     return strcpy(dst, src);
 }
 
-U032 os_string_length(const U008* s)
+U032 NV_API_CALL os_string_length(const U008* s)
 {
     return strlen(s);
 }
 
-RM_STATUS os_strncpy_from_user(
+RM_STATUS NV_API_CALL os_strncpy_from_user(
     U008 *dst,
     const U008 *src,
     U032 n
@@ -532,12 +537,12 @@ RM_STATUS os_strncpy_from_user(
     return copyinstr(src, dst, n, NULL) ? RM_ERR_INVALID_POINTER : RM_OK;
 }
 
-U032 os_get_page_size(void)
+U032 NV_API_CALL os_get_page_size(void)
 {
     return PAGE_SIZE;
 }
 
-NvU64 os_get_page_mask(void)
+NvU64 NV_API_CALL os_get_page_mask(void)
 {
     /*
      * On FreeBSD, PAGE_MASK means (PAGE_SIZE - 1); on Linux it means the
@@ -547,133 +552,145 @@ NvU64 os_get_page_mask(void)
     return ~PAGE_MASK;
 }
 
-NvU64 os_get_system_memory_size(void)
+NvU64 NV_API_CALL os_get_system_memory_size(void)
 {
     return ((NvU64)physmem * PAGE_SIZE) / RM_PAGE_SIZE;
 }
 
-U032 os_get_cpu_count(void)
+U032 NV_API_CALL os_get_cpu_count(void)
 {
     return mp_ncpus;
 }
 
-RM_STATUS os_flush_cpu_cache(void)
+RM_STATUS NV_API_CALL os_flush_cpu_cache(void)
 {
     /*
-     * XXX This will do for now, but this may need to be extended to make
-     * IPI calls (flushing all caches).
+     * XXX This will do for now, but this may need to be extended
+     * to make IPI calls (flushing all caches).
      */
     __asm__ __volatile__("wbinvd": : :"memory");
     return RM_OK;
 }
 
-void os_flush_cpu_write_combine_buffer(void)
+void NV_API_CALL os_flush_cpu_write_combine_buffer(void)
 {
     __asm__ __volatile__("sfence": : :"memory");
 }
 
-RM_STATUS os_raise_smp_barrier(void)
+RM_STATUS NV_API_CALL os_raise_smp_barrier(void)
 {
     return RM_OK;
 }
 
-RM_STATUS os_clear_smp_barrier(void)
+RM_STATUS NV_API_CALL os_clear_smp_barrier(void)
 {
     return RM_OK;
 }
 
 struct os_mutex {
+    nv_stack_t *sp;
     struct mtx mutex_mtx;
-    /* XXX lockmgr broken w/ 5.2-current KSE */
-    struct cv mutex_wait;
+    struct cv mutex_cv;
     int refcnt;
 };
 
-RM_STATUS os_alloc_sema(void **semaphore)
+RM_STATUS NV_API_CALL os_alloc_sema(void **semaphore)
 {
+    nv_stack_t *sp;
     struct os_mutex *mtx;
     RM_STATUS status;
 
-    if ((status = os_alloc_mem((void **) &mtx, sizeof(struct os_mutex))
-            != RM_OK))
+    NV_UMA_ZONE_ALLOC_STACK(sp);
+    if (sp == NULL)
+        return RM_ERR_NO_FREE_MEM;
+
+    status = os_alloc_mem((void **)&mtx, sizeof(struct os_mutex));
+    if (status != RM_OK) {
+        NV_UMA_ZONE_FREE_STACK(sp);
         return status;
+    }
 
-    mtx_init(&mtx->mutex_mtx, "rm.mutex_mtx", NULL, MTX_SPIN | MTX_RECURSE);
-    cv_init(&mtx->mutex_wait, "rm.mutex_wait");
+    mtx_init(&mtx->mutex_mtx, "os.mutex_mtx", NULL, MTX_DEF | MTX_RECURSE);
+    cv_init(&mtx->mutex_cv, "os.mutex_cv");
 
+    mtx->sp = sp;
     mtx->refcnt = 1;
+
     *semaphore = (void *) mtx;
 
     return RM_OK;
 }
 
-RM_STATUS os_free_sema(void *semaphore)
+RM_STATUS NV_API_CALL os_free_sema(void *semaphore)
 {
     struct os_mutex *mtx = semaphore;
+    nv_stack_t *sp;
+
+    sp = mtx->sp;
+    NV_UMA_ZONE_FREE_STACK(sp);
 
     mtx_destroy(&mtx->mutex_mtx);
-    /* XXX lockmgr broken w/ 5.2-current KSE */
-    cv_destroy(&mtx->mutex_wait);
+    cv_destroy(&mtx->mutex_cv);
 
     os_free_mem(semaphore);
 
     return RM_OK;
 }
 
-RM_STATUS os_acquire_sema(void *semaphore)
+RM_STATUS NV_API_CALL os_acquire_sema(void *semaphore)
 {
     struct os_mutex *mtx = semaphore;
 
-    mtx_lock_spin(&mtx->mutex_mtx);
+    mtx_lock(&mtx->mutex_mtx);
     if (mtx->refcnt > 0)
-        rm_disable_interrupts(NULL);
+        rm_disable_interrupts(mtx->sp);
     mtx->refcnt--;
     if (mtx->refcnt < 0)
-        cv_wait(&mtx->mutex_wait, &mtx->mutex_mtx);
-    mtx_unlock_spin(&mtx->mutex_mtx);
+        cv_wait(&mtx->mutex_cv, &mtx->mutex_mtx);
+    mtx_unlock(&mtx->mutex_mtx);
 
     return RM_OK;
 }
 
-BOOL os_cond_acquire_sema(void *semaphore)
+BOOL NV_API_CALL os_cond_acquire_sema(void *semaphore)
 {
     struct os_mutex *mtx = semaphore;
 
-    mtx_lock_spin(&mtx->mutex_mtx);
+    mtx_lock(&mtx->mutex_mtx);
     if (mtx->refcnt < 1) {
-        mtx_unlock_spin(&mtx->mutex_mtx);
+        mtx_unlock(&mtx->mutex_mtx);
         return FALSE;
     } else {
-        rm_disable_interrupts(NULL);
+        rm_disable_interrupts(mtx->sp);
         mtx->refcnt--;
-        mtx_unlock_spin(&mtx->mutex_mtx);
+        mtx_unlock(&mtx->mutex_mtx);
     }
 
     return TRUE;
 }
 
-RM_STATUS os_release_sema(void *semaphore)
+RM_STATUS NV_API_CALL os_release_sema(void *semaphore)
 {
     struct os_mutex *mtx = semaphore;
 
-    mtx_lock_spin(&mtx->mutex_mtx);
+    mtx_lock(&mtx->mutex_mtx);
     if (mtx->refcnt < 0)
-        cv_signal(&mtx->mutex_wait);
-    if (!mtx->refcnt)
-        rm_enable_interrupts(NULL);
+        cv_signal(&mtx->mutex_cv);
+    else
+        rm_enable_interrupts(mtx->sp);
     mtx->refcnt++;
-    mtx_unlock_spin(&mtx->mutex_mtx);
+    mtx_unlock(&mtx->mutex_mtx);
 
     return RM_OK;
 }
 
-BOOL os_is_acquired_sema(void *semaphore)
+BOOL NV_API_CALL os_is_acquired_sema(void *semaphore)
 {
     struct os_mutex *mtx = semaphore;
     return (mtx->refcnt < 1);
 }
 
-BOOL os_pat_supported(void)
+BOOL NV_API_CALL os_pat_supported(void)
 {
     /*
      * FreeBSD has no native PAT support and there's no good
@@ -682,7 +699,7 @@ BOOL os_pat_supported(void)
     return FALSE;
 }
 
-void* NV_STACKWATCH_CALLBACK os_get_stack_start(void *stack_pointer)
+void* NV_API_CALL NV_STACKWATCH_CALLBACK os_get_stack_start(void *stack_pointer)
 {
     struct thread *td;
 #if defined(NVCPU_X86_64)
@@ -693,17 +710,17 @@ void* NV_STACKWATCH_CALLBACK os_get_stack_start(void *stack_pointer)
     return (void *)cpu_getstack(td);
 }
 
-NvU64 os_get_current_pdpte(U032 address)
+NvU64 NV_API_CALL os_get_current_pdpte(U032 address)
 {
     return 0;
 }
 
-RM_STATUS os_set_mlock_capability()
+RM_STATUS NV_API_CALL os_set_mlock_capability()
 {
     return (RM_ERROR);
 }
 
-S032 os_mlock_user_memory(
+S032 NV_API_CALL os_mlock_user_memory(
     void *address,
     U032 length
 )
@@ -711,7 +728,7 @@ S032 os_mlock_user_memory(
     return -1;
 }
 
-S032 os_munlock_user_memory(
+S032 NV_API_CALL os_munlock_user_memory(
     void *address,
     U032 length
 )
@@ -726,10 +743,27 @@ RM_STATUS NV_API_CALL os_check_process_map_limit(
     return (RM_ERROR);
 }
 
-void NV_API_CALL os_register_compatible_ioctl(U032 cmd, U032 size)
+void NV_API_CALL os_register_compatible_ioctl(
+    U032 cmd,
+    U032 size
+)
 {
 }
 
-void NV_API_CALL os_unregister_compatible_ioctl(U032 cmd, U032 size)
+void NV_API_CALL os_unregister_compatible_ioctl(
+    U032 cmd,
+    U032 size
+)
 {
 }
+
+RM_STATUS NV_API_CALL os_disable_console_access(void)
+{
+    return RM_OK;
+}
+
+RM_STATUS NV_API_CALL os_enable_console_access(void)
+{
+    return RM_OK;
+}
+
