@@ -17,6 +17,9 @@
 #include <machine/vm86.h>
 #endif
 
+uma_zone_t nvidia_stack_t_zone;
+static nv_stack_t *__nvidia_init_sp = NULL;
+
 devclass_t nvidia_devclass;
 nv_state_t nvidia_ctl_state;
 
@@ -88,6 +91,7 @@ fail:
 #ifdef NV_SUPPORT_ACPI_PM
 int nvidia_suspend(device_t dev)
 {
+    nv_stack_t *sp;
     struct nvidia_softc *sc;
     nv_state_t *nv;
     int status = RM_ERROR;
@@ -96,26 +100,39 @@ int nvidia_suspend(device_t dev)
     if (devclass_get_softc(devclass_find("ACPI"), 0) == NULL)
         return ENODEV;
 
+    NV_UMA_ZONE_ALLOC_STACK(sp);
+    if (sp == NULL)
+        return ENOMEM;
+
     sc = device_get_softc(dev);
     nv = sc->nv_state;
 
-    NV_PCI_CHECK_CONFIG_SPACE(nv, TRUE);
-    status = rm_power_management(NULL, nv, 0, NV_PM_ACPI_STANDBY);
+    NV_PCI_CHECK_CONFIG_SPACE(sp, nv, TRUE, TRUE, TRUE);
+    status = rm_power_management(sp, nv, 0, NV_PM_ACPI_STANDBY);
+
+    NV_UMA_ZONE_FREE_STACK(sp);
 
     return status;
 }
 
 int nvidia_resume(device_t dev)
 {
+    nv_stack_t *sp;
     struct nvidia_softc *sc;
     nv_state_t *nv;
     int status = RM_ERROR;
 
+    NV_UMA_ZONE_ALLOC_STACK(sp);
+    if (sp == NULL)
+        return ENOMEM;
+
     sc = device_get_softc(dev);
     nv = sc->nv_state;
 
-    NV_PCI_CHECK_CONFIG_SPACE(nv, TRUE);
-    status = rm_power_management(NULL, nv, 0, NV_PM_ACPI_RESUME);
+    NV_PCI_CHECK_CONFIG_SPACE(sp, nv, TRUE, TRUE, TRUE);
+    status = rm_power_management(sp, nv, 0, NV_PM_ACPI_RESUME);
+
+    NV_UMA_ZONE_FREE_STACK(sp);
 
     return status;
 }
@@ -187,40 +204,25 @@ void nvidia_intr(void *xsc)
     struct nvidia_softc *sc;
     nv_state_t *nv;
     U032 run_bottom_half = 0;
+    nv_stack_t *sp;
 
     sc = (struct nvidia_softc *) xsc;
     nv = sc->nv_state;
 
-    NV_PCI_CHECK_CONFIG_SPACE(nv, TRUE);
-    rm_isr(NULL, nv, &run_bottom_half);
+    sp = sc->isr_sp;
+
+    if (sp == NULL)
+        return;
+
+    NV_PCI_CHECK_CONFIG_SPACE(sp, nv, TRUE, TRUE, FALSE);
+    rm_isr(sp, nv, &run_bottom_half);
 
     if (run_bottom_half) {
         /* We're not executing in an HW ISR context */
-        rm_isr_bh(NULL, nv);
+        rm_isr_bh(sp, nv);
     }
 }
 
-int nvidia_post_vbios(struct nv_ioctl_post_vbios *pv)
-{
-    unsigned int i;
-    struct nvidia_softc *sc;
-    nv_state_t *nv;
-
-    for (i = 0; i < NV_MAX_DEVICES; i++) {
-        sc = devclass_get_softc(nvidia_devclass, i);
-        if (!sc)
-            continue;
-        nv = sc->nv_state;
-
-        if ((nv->bus == pv->bus) && (nv->slot == pv->slot) &&
-                ((nv->flags & NV_FLAG_WAS_POSTED) == 0) &&
-                ((nv->flags & NV_FLAG_OPEN) == 0))
-            nv->flags |= NV_FLAG_NEEDS_POSTING;
-    }
-
-    return 0;
-}
-
 int nvidia_get_card_info(struct nv_ioctl_card_info *ci)
 {
     unsigned int i;
@@ -301,10 +303,11 @@ int nvidia_handle_ioctl(
     struct nvidia_softc *sc;
     nv_state_t *nv;
     int unit = minor(dev);
+    nv_stack_t *sp;
 
     if (unit == CDEV_CTL_MINOR) {
-        /* the control device is "special" */
         nv = &nvidia_ctl_state;
+        sc = nv->os_state;
     } else {
         sc = devclass_get_softc(nvidia_devclass, unit);
         if (!sc)
@@ -312,9 +315,11 @@ int nvidia_handle_ioctl(
         nv = sc->nv_state;
     }
 
-    NV_PCI_CHECK_CONFIG_SPACE(nv, TRUE);
+    sp = sc->api_sp;
 
-    if (rm_ioctl(NULL, nv, __TD_FDT(td), __NV_IOC_NR(cmd), data))
+    NV_PCI_CHECK_CONFIG_SPACE(sp, nv, TRUE, TRUE, TRUE);
+
+    if (rm_ioctl(sp, nv, __TD_FDT(td), __NV_IOC_NR(cmd), data))
         return 0;
 
     return EINVAL;
@@ -328,6 +333,7 @@ int nvidia_open_ctl(
 {
     struct nvidia_softc *sc;
     nv_state_t *nv = &nvidia_ctl_state;
+    BOOL new_filep = FALSE;
     struct nvidia_filep *filep;
 
     sc = nv->os_state;
@@ -344,13 +350,21 @@ int nvidia_open_ctl(
         filep->fd_table = __TD_FDT(td);
         filep->fd_refcnt = 0;
         STAILQ_INSERT_HEAD(&sc->filep_queue, filep, queue);
+        new_filep = TRUE;
     }
 
     filep->fd_refcnt++;
 
     if (sc->refcnt == 0) {
+        NV_UMA_ZONE_ALLOC_STACK(sc->api_sp);
+        if (sc->api_sp == NULL) {
+            if (new_filep)
+                free(filep, M_NVIDIA);
+            return ENOMEM;
+        }
+
+        STAILQ_INIT(&sc->event_queue);
         nv->flags |= (NV_FLAG_OPEN | NV_FLAG_CONTROL);
-        nv->flags &= ~NV_FLAG_HOTKEY_OCCURRED;
     }
 
     sc->refcnt++;
@@ -365,7 +379,9 @@ int nvidia_close_ctl(
 {
     struct nvidia_softc *sc;
     nv_state_t *nv = &nvidia_ctl_state;
+    struct nvidia_event *et;
     struct nvidia_filep *filep;
+    nv_stack_t *sp;
 
     sc = nv->os_state;
 
@@ -377,14 +393,24 @@ int nvidia_close_ctl(
     if (filep == NULL)
         return EINVAL;
 
+    sp = sc->api_sp;
+
     if (--filep->fd_refcnt == 0) {
-        rm_free_unused_clients(NULL, nv, __TD_FDT(td));
+        rm_free_unused_clients(sp, nv, __TD_FDT(td));
         STAILQ_REMOVE(&sc->filep_queue, filep, nvidia_filep, queue);
         free(filep, M_NVIDIA);
     }
 
-    if (--sc->refcnt == 0)
-        nv->flags = 0;
+    if (--sc->refcnt == 0) {
+        while ((et = STAILQ_FIRST(&sc->event_queue))) {
+            STAILQ_REMOVE(&sc->event_queue, et, nvidia_event, queue);
+            free(et, M_NVIDIA);
+        }
+
+        NV_UMA_ZONE_FREE_STACK(sp);
+
+        nv->flags &= ~NV_FLAG_OPEN;
+    }
 
     return 0;
 }
@@ -395,9 +421,11 @@ int nvidia_open_dev(
     d_thread_t *td
 )
 {
+    int status = ENOMEM;
     nv_state_t *nv = sc->nv_state;
     BOOL new_filep = FALSE;
     struct nvidia_filep *filep;
+    nv_stack_t *sp = NULL;
 
     STAILQ_FOREACH(filep, &sc->filep_queue, queue) {
         if (filep->fd_table == __TD_FDT(td)) 
@@ -413,14 +441,32 @@ int nvidia_open_dev(
         new_filep = TRUE;
     }
 
-    NV_PCI_CHECK_CONFIG_SPACE(nv, TRUE);
+    if (sc->refcnt == 0) {
+        NV_UMA_ZONE_ALLOC_STACK(sc->api_sp);
+        if (sc->api_sp == NULL)
+            goto failed;
+
+        NV_UMA_ZONE_ALLOC_STACK(sc->pci_cfgchk_sp);
+        if (sc->pci_cfgchk_sp == NULL)
+            goto failed;
+
+        NV_UMA_ZONE_ALLOC_STACK(sc->isr_sp);
+        if (sc->isr_sp == NULL)
+            goto failed;
+
+        NV_UMA_ZONE_ALLOC_STACK(sc->timer_sp);
+        if (sc->timer_sp == NULL)
+            goto failed;
+    }
+
+    sp = sc->api_sp;
+    NV_PCI_CHECK_CONFIG_SPACE(sp, nv, TRUE, TRUE, TRUE);
 
     if (sc->refcnt == 0) {
-        if (!rm_init_adapter(NULL, nv)) {
+        if (!rm_init_adapter(sp, nv)) {
             device_printf(sc->dev, "NVRM: rm_init_adapter() failed!\n");
-            if (new_filep)
-                free(filep, M_NVIDIA);
-            return EIO;
+            status = EIO;
+            goto failed;
         }
 
         STAILQ_INIT(&sc->event_queue);
@@ -434,6 +480,34 @@ int nvidia_open_dev(
     sc->refcnt++;
 
     return 0;
+
+failed:
+    if (sc->refcnt == 0) {
+        if (sc->timer_sp != NULL) {
+            NV_UMA_ZONE_FREE_STACK(sc->timer_sp);
+            sc->timer_sp = NULL;
+        }
+
+        if (sc->isr_sp != NULL) {
+            NV_UMA_ZONE_FREE_STACK(sc->isr_sp);
+            sc->isr_sp = NULL;
+        }
+
+        if (sc->pci_cfgchk_sp != NULL) {
+            NV_UMA_ZONE_FREE_STACK(sc->pci_cfgchk_sp);
+            sc->pci_cfgchk_sp = NULL;
+        }
+
+        if (sc->api_sp != NULL) {
+            NV_UMA_ZONE_FREE_STACK(sc->api_sp);
+            sc->api_sp = NULL;
+        }
+    }
+
+    if (new_filep)
+        free(filep, M_NVIDIA);
+
+    return status;
 }
 
 int nvidia_close_dev(
@@ -445,6 +519,7 @@ int nvidia_close_dev(
     nv_state_t *nv = sc->nv_state;
     struct nvidia_event *et;
     struct nvidia_filep *filep;
+    nv_stack_t *sp;
 
     STAILQ_FOREACH(filep, &sc->filep_queue, queue) {
         if (filep->fd_table == __TD_FDT(td)) 
@@ -454,23 +529,37 @@ int nvidia_close_dev(
     if (filep == NULL)
         return EINVAL;
 
-    NV_PCI_CHECK_CONFIG_SPACE(nv, TRUE);
+    sp = sc->api_sp;
+
+    NV_PCI_CHECK_CONFIG_SPACE(sp, nv, TRUE, TRUE, TRUE);
 
     if (--filep->fd_refcnt == 0) {
-        rm_free_unused_clients(NULL, nv, __TD_FDT(td));
+        rm_free_unused_clients(sp, nv, __TD_FDT(td));
         STAILQ_REMOVE(&sc->filep_queue, filep, nvidia_filep, queue);
         free(filep, M_NVIDIA);
     }
 
     if (--sc->refcnt == 0) {
-        rm_disable_adapter(NULL, nv);
-        rm_shutdown_adapter(NULL, nv);
+        rm_disable_adapter(sp, nv);
+        rm_shutdown_adapter(sp, nv);
 
         while ((et = STAILQ_FIRST(&sc->event_queue))) {
             STAILQ_REMOVE(&sc->event_queue, et, nvidia_event, queue);
             free(et, M_NVIDIA);
         }
 
+        NV_UMA_ZONE_FREE_STACK(sc->timer_sp);
+        sc->timer_sp = NULL;
+
+        NV_UMA_ZONE_FREE_STACK(sc->isr_sp);
+        sc->isr_sp = NULL;
+
+        NV_UMA_ZONE_FREE_STACK(sc->pci_cfgchk_sp);
+        sc->pci_cfgchk_sp = NULL;
+
+        NV_UMA_ZONE_FREE_STACK(sc->api_sp);
+        sc->api_sp = NULL;
+
         nv->flags &= ~NV_FLAG_OPEN;
     }
 
@@ -486,6 +575,7 @@ int nvidia_modevent(
 {
     nv_state_t *nv;
     struct nvidia_softc *sc;
+    nv_stack_t *sp;
 
     switch (what) {
         case MOD_LOAD:
@@ -497,14 +587,29 @@ int nvidia_modevent(
              */
             sc = &nvidia_ctl_sc;
 
+            nvidia_stack_t_zone = uma_zcreate("nv_stack_t", sizeof(nv_stack_t),
+                    NULL, NULL, NULL, NULL, UMA_ALIGN_PTR, 0);
+            if (nvidia_stack_t_zone == NULL)
+                return ENOMEM;
+
+            NV_UMA_ZONE_ALLOC_STACK(sp);
+            if (sp == NULL) {
+                uma_zdestroy(nvidia_stack_t_zone);
+                return ENOMEM;
+            }
+
             bzero(sc, sizeof(nvidia_softc_t));
             STAILQ_INIT(&sc->filep_queue);
 
-            if (!rm_init_rm(NULL)) {
+            if (!rm_init_rm(sp)) {
                 printf("NVRM: rm_init_rm() failed!\n");
+                NV_UMA_ZONE_FREE_STACK(sp);
+                uma_zdestroy(nvidia_stack_t_zone);
                 return EIO;
             }
 
+            __nvidia_init_sp = sp;
+
             mtx_init(&sc->rm_mtx, "ctl.rm_mtx", NULL, MTX_SPIN | MTX_RECURSE);
             sx_init(&sc->api_sx, "ctl.api_sx");
 
@@ -537,11 +642,16 @@ int nvidia_modevent(
             mtx_destroy(&sc->rm_mtx);
             sx_destroy(&sc->api_sx);
 
-            rm_shutdown_rm(NULL);
+            sp = __nvidia_init_sp;
+            rm_shutdown_rm(sp);
+
+            NV_UMA_ZONE_FREE_STACK(sp);
 
             nvidia_sysctl_exit();
             nvidia_linux_exit();
 
+            uma_zdestroy(nvidia_stack_t_zone);
+
             break;
 
         default:
@@ -554,6 +664,7 @@ int nvidia_modevent(
 
 #ifdef NV_SUPPORT_OS_AGP
 S032 nv_os_agp_init(
+    nv_stack_t *sp,
     nv_state_t *nv,
     void **base,
     U032 *limit
@@ -593,9 +704,9 @@ S032 nv_os_agp_init(
         return -EIO;
     }
 
-    rm_read_registry_dword(NULL, NULL, "NVreg", "ReqAGPRate", &rate);
-    rm_read_registry_dword(NULL, NULL, "NVreg", "EnableAGPFW", &fw);
-    rm_read_registry_dword(NULL, NULL, "NVreg", "EnableAGPSBA", &sba);
+    rm_read_registry_dword(sp, NULL, "NVreg", "ReqAGPRate", &rate);
+    rm_read_registry_dword(sp, NULL, "NVreg", "EnableAGPFW", &fw);
+    rm_read_registry_dword(sp, NULL, "NVreg", "EnableAGPSBA", &sba);
 
     if (AGP_MODE_GET_MODE_3(mode))
         rate = (rate >> 2) & 3;
@@ -617,8 +728,8 @@ S032 nv_os_agp_init(
     }
 
     size = ai.ai_aperture_size / RM_PAGE_SIZE / 8;
-    
-    if (os_alloc_mem(&bitmap, size) != RM_OK) {
+
+    if (os_alloc_mem((void **)&bitmap, size) != RM_OK) {
         agp_release(sc->agp_dev);
         os_unset_mem_range(ai.ai_aperture_base, ai.ai_aperture_size);
         return -EIO;
@@ -626,7 +737,7 @@ S032 nv_os_agp_init(
 
     os_mem_set(bitmap, 0xff, size);
 
-    if (rm_set_agp_bitmap(NULL, nv, bitmap) != RM_OK) {
+    if (rm_set_agp_bitmap(sp, nv, bitmap) != RM_OK) {
         agp_release(sc->agp_dev);
         os_free_mem(bitmap);
         os_unset_mem_range(ai.ai_aperture_base, ai.ai_aperture_size);
@@ -639,7 +750,10 @@ S032 nv_os_agp_init(
     return 0;
 }
 
-S032 nv_os_agp_teardown(nv_state_t *nv)
+S032 nv_os_agp_teardown(
+    nv_stack_t *sp,
+    nv_state_t *nv
+)
 {
     struct nvidia_softc *sc = nv->os_state;
     void *bitmap;
@@ -647,7 +761,7 @@ S032 nv_os_agp_teardown(nv_state_t *nv)
     if (agp_release(sc->agp_dev) != 0)
         return -EBUSY;
 
-    rm_clear_agp_bitmap(NULL, nv, &bitmap);
+    rm_clear_agp_bitmap(sp, nv, &bitmap);
     os_free_mem(bitmap);
 
     os_unset_mem_range(nv->agp.address, nv->agp.size);
@@ -656,7 +770,7 @@ S032 nv_os_agp_teardown(nv_state_t *nv)
 }
 #endif /* NV_SUPPORT_OS_AGP */
 
-RM_STATUS nv_agp_init(
+RM_STATUS NV_API_CALL nv_agp_init(
     nv_state_t *nv,
     void **base,
     void *limit,
@@ -664,6 +778,7 @@ RM_STATUS nv_agp_init(
 )
 {
     RM_STATUS status = RM_ERROR;
+    nv_stack_t *sp;
 
     if (NV_AGP_ENABLED(nv))
         return RM_ERR_STATE_IN_USE;
@@ -678,16 +793,23 @@ RM_STATUS nv_agp_init(
         return RM_OK;
     }
 
+    NV_UMA_ZONE_ALLOC_STACK(sp);
+    if (sp == NULL)
+        return RM_ERR_NO_FREE_MEM;
+
 #ifdef NV_SUPPORT_OS_AGP
     if ((config & NVOS_AGP_CONFIG_OSAGP) != 0) {
-        if (nv_os_agp_init(nv, base, limit) == 0) {
+        if (nv_os_agp_init(sp, nv, base, limit) == 0) {
             /*
              * If the operating system AGP GART driver successfully
              * configured its backend, apply chipset overrides.
              */
-            rm_update_agp_config(NULL, nv);
+            rm_update_agp_config(sp, nv);
+            NV_UMA_ZONE_FREE_STACK(sp);
+
             nv->agp_config = NVOS_AGP_CONFIG_OSAGP;
             nv->agp_status = NV_AGP_STATUS_ENABLED;
+
             return RM_OK;
         }
     }
@@ -712,33 +834,46 @@ RM_STATUS nv_agp_init(
         goto failed;
     }
 
-    status = rm_init_agp(NULL, nv);
+    status = rm_init_agp(sp, nv);
     if (status == RM_OK) {
+        NV_UMA_ZONE_FREE_STACK(sp);
+
         nv->agp_config = NVOS_AGP_CONFIG_NVAGP;
         nv->agp_status = NV_AGP_STATUS_ENABLED;
+
         return status;
     }
 
 failed:
+    NV_UMA_ZONE_FREE_STACK(sp);
+
     nv->agp_config = NVOS_AGP_CONFIG_DISABLE_AGP;
     nv->agp_status = NV_AGP_STATUS_FAILED;
 
     return status;
 }
 
-RM_STATUS nv_agp_teardown(nv_state_t *nv)
+RM_STATUS NV_API_CALL nv_agp_teardown(nv_state_t *nv)
 {
     RM_STATUS status = RM_ERR_NOT_SUPPORTED;
+    nv_stack_t *sp;
 
     if (!NV_AGP_ENABLED(nv))
         return status;
 
+    NV_UMA_ZONE_ALLOC_STACK(sp);
+    if (sp == NULL)
+        return RM_ERR_NO_FREE_MEM;
+
 #ifdef NV_SUPPORT_OS_AGP
     if (NV_OSAGP_ENABLED(nv))
-        status = (nv_os_agp_teardown(nv) == 0) ? RM_OK : RM_ERROR;
+        status = (nv_os_agp_teardown(sp, nv) == 0)
+            ? RM_OK : RM_ERROR;
 #endif
     if (NV_NVAGP_ENABLED(nv))
-        status = rm_teardown_agp(NULL, nv);
+        status = rm_teardown_agp(sp, nv);
+
+    NV_UMA_ZONE_FREE_STACK(sp);
 
     nv->agp_config = NVOS_AGP_CONFIG_DISABLE_AGP;
     nv->agp_status = NV_AGP_STATUS_DISABLED;
@@ -746,7 +881,7 @@ RM_STATUS nv_agp_teardown(nv_state_t *nv)
     return status;
 }
 
-S032 nv_no_incoherent_mappings(void)
+S032 NV_API_CALL nv_no_incoherent_mappings(void)
 {
 #if !defined(NVCPU_X86_64) && !defined(NV_SUPPORT_OS_AGP)
     return 1;
@@ -765,7 +900,7 @@ S032 nv_no_incoherent_mappings(void)
 #endif
 }
 
-void nv_lock_rm(nv_state_t *nv)
+void NV_API_CALL nv_lock_rm(nv_state_t *nv)
 {
     /*
      * With SMPng, the "giant" kernel lock is gone. That means that we're
@@ -779,7 +914,7 @@ void nv_lock_rm(nv_state_t *nv)
     mtx_lock_spin(&sc->rm_mtx);
 }
 
-void nv_unlock_rm(nv_state_t *nv)
+void NV_API_CALL nv_unlock_rm(nv_state_t *nv)
 {
     struct nvidia_softc *sc = nv->os_state;
     mtx_unlock_spin(&sc->rm_mtx);
@@ -798,7 +933,7 @@ void nv_unlock_api(nv_state_t *nv)
 }
 
 
-void nv_post_event(
+void NV_API_CALL nv_post_event(
     nv_state_t *nv,
     nv_event_t *event,
     U032 hObject,
@@ -823,11 +958,10 @@ void nv_post_event(
 
     nv_unlock_rm(nv);
 
-    /* XXX Fix me? (os events) */
     selwakeup(&sc->rsel);
 }
 
-S032 nv_get_event(
+S032 NV_API_CALL nv_get_event(
     nv_state_t *nv,
     void *file,
     nv_event_t *event,
@@ -868,7 +1002,7 @@ S032 nv_get_event(
     return RM_ERROR; /* RM polling? */
 }
 
-void* nv_alloc_kernel_mapping(
+void* NV_API_CALL nv_alloc_kernel_mapping(
     nv_state_t *nv,
     NvU64 address,
     U032 size,
@@ -894,7 +1028,7 @@ void* nv_alloc_kernel_mapping(
     return NULL;
 }
 
-S032 nv_free_kernel_mapping(
+S032 NV_API_CALL nv_free_kernel_mapping(
     nv_state_t *nv,
     void *address,
     void *private
@@ -1104,7 +1238,7 @@ S032 nv_free_agp_pages(
 #endif /* NV_SUPPORT_OS_AGP */
 
 
-RM_STATUS nv_alloc_pages(
+RM_STATUS NV_API_CALL nv_alloc_pages(
     nv_state_t *nv,
     U032 count,
     U032 alloc_type_agp,
@@ -1116,33 +1250,42 @@ RM_STATUS nv_alloc_pages(
 {
     U032 offset;
     RM_STATUS status = RM_ERR_NO_FREE_MEM;
+    nv_stack_t *sp = NULL;
 
     if (alloc_type_agp) {
         if (!NV_AGP_ENABLED(nv))
             return RM_ERR_NOT_SUPPORTED;
 
+        NV_UMA_ZONE_ALLOC_STACK(sp);
+        if (sp == NULL)
+            return RM_ERR_NO_FREE_MEM;
+
 #ifdef NV_SUPPORT_OS_AGP
         if (NV_OSAGP_ENABLED(nv)) {
-            status = rm_alloc_agp_bitmap(NULL, nv, count, &offset);
+            status = rm_alloc_agp_bitmap(sp, nv, count, &offset);
             if (status != RM_OK)
                 goto failed;
 
             if (nv_alloc_agp_pages(nv, count, (offset << PAGE_SHIFT),
                     private) != 0) {
-                rm_free_agp_bitmap(NULL, nv, count, offset);
+                rm_free_agp_bitmap(sp, nv, count, offset);
                 goto failed;
             }
 
+            NV_UMA_ZONE_FREE_STACK(sp);
+
             pte_array[0] = (nv->agp.address + (offset << PAGE_SHIFT));
             return RM_OK;
         }
 #endif /* NV_SUPPORT_OS_AGP */
 
         if (NV_NVAGP_ENABLED(nv)) {
-            status = rm_alloc_agp_pages(NULL, nv, count, private, &offset);
+            status = rm_alloc_agp_pages(sp, nv, count, private, &offset);
             if (status != RM_OK)
                 goto failed;
 
+            NV_UMA_ZONE_FREE_STACK(sp);
+
             pte_array[0] = (nv->agp.address + (offset << PAGE_SHIFT));
             return RM_OK;
         }
@@ -1165,10 +1308,13 @@ RM_STATUS nv_alloc_pages(
     }
 
 failed:
+    if (sp != NULL)
+        NV_UMA_ZONE_FREE_STACK(sp);
+
     return status;
 }
 
-RM_STATUS nv_free_pages(
+RM_STATUS NV_API_CALL nv_free_pages(
     nv_state_t *nv,
     U032 count,
     U032 alloc_type_agp,
@@ -1178,11 +1324,16 @@ RM_STATUS nv_free_pages(
 )
 {
     RM_STATUS status = RM_ERROR;
+    nv_stack_t *sp = NULL;
 
     if (alloc_type_agp) {
         if (!NV_AGP_ENABLED(nv))
             return RM_ERR_NOT_SUPPORTED;
 
+        NV_UMA_ZONE_ALLOC_STACK(sp);
+        if (sp == NULL)
+            return RM_ERR_NO_FREE_MEM;
+
 #ifdef NV_SUPPORT_OS_AGP
         if (NV_OSAGP_ENABLED(nv)) {
             U032 offset;
@@ -1190,15 +1341,19 @@ RM_STATUS nv_free_pages(
             if (nv_free_agp_pages(nv, count, private, &offset) != 0)
                 goto failed;
 
-            rm_free_agp_bitmap(NULL, nv, count, (offset >> PAGE_SHIFT));
+            rm_free_agp_bitmap(sp, nv, count, (offset >> PAGE_SHIFT));
+            NV_UMA_ZONE_FREE_STACK(sp);
+
             return RM_OK;
         }
 #endif /* NV_SUPPORT_OS_AGP */
 
         if (NV_NVAGP_ENABLED(nv)) {
-            if (rm_free_agp_pages(NULL, nv, private) != RM_OK)
+            if (rm_free_agp_pages(sp, nv, private) != RM_OK)
                 goto failed;
         }
+
+        NV_UMA_ZONE_FREE_STACK(sp);
     } else {
         if (!alloc_type_contiguous) {
             if (nv_free_system_pages(nv, private))
@@ -1212,10 +1367,13 @@ RM_STATUS nv_free_pages(
     return RM_OK;
 
 failed:
+    if (sp != NULL)
+        NV_UMA_ZONE_FREE_STACK(sp);
+
     return status;
 }
 
-NvU64 nv_dma_to_mmap_token(
+NvU64 NV_API_CALL nv_dma_to_mmap_token(
     nv_state_t *nv,
     NvU64 address
 )
@@ -1249,7 +1407,7 @@ NvU64 nv_dma_to_mmap_token(
 }
 
 
-NvU64 nv_get_kern_phys_address(NvU64 address)
+NvU64 NV_API_CALL nv_get_kern_phys_address(NvU64 address)
 {
     vm_offset_t va = (vm_offset_t) address;
 
@@ -1266,7 +1424,7 @@ NvU64 nv_get_kern_phys_address(NvU64 address)
     return vtophys(va);
 }
 
-NvU64 nv_get_user_phys_address(NvU64 address)
+NvU64 NV_API_CALL nv_get_user_phys_address(NvU64 address)
 {
     struct vmspace *vm;
     vm_offset_t va = (vm_offset_t) address;
@@ -1292,8 +1450,11 @@ int nvidia_mmap_dev(
 {
     struct nvidia_alloc *at;
     nv_state_t *nv = sc->nv_state;
+    nv_stack_t *sp;
 
-    NV_PCI_CHECK_CONFIG_SPACE(nv, TRUE);
+    sp = sc->api_sp;
+
+    NV_PCI_CHECK_CONFIG_SPACE(sp, nv, TRUE, TRUE, TRUE);
 
     /*
      * Offsets that fall into the frame buffer, registry or AGP
@@ -1331,22 +1492,24 @@ int nvidia_mmap_dev(
 
 void nvidia_rc_timer(void *data)
 {
-    nv_state_t *nv = (nv_state_t *) data;
-    struct nvidia_softc *sc;
-    
-    NV_PCI_CHECK_CONFIG_SPACE(nv, TRUE);
+    nv_state_t *nv = data;
+    struct nvidia_softc *sc = nv->os_state;
+    nv_stack_t *sp;
+
+    sp = sc->timer_sp;
+
+    NV_PCI_CHECK_CONFIG_SPACE(sp, nv, TRUE, TRUE, FALSE);
 
     /*
      * We need this timer to trigger again one second from
      * now, reset the timeout.
      */
-    rm_run_rc_callback(NULL, nv);
+    rm_run_rc_callback(sp, nv);
 
-    sc = nv->os_state;
     sc->timer_ch = timeout(nvidia_rc_timer, (void *) nv, hz);
 }
 
-int nv_start_rc_timer(
+int NV_API_CALL nv_start_rc_timer(
     nv_state_t *nv
 )
 {
@@ -1361,7 +1524,7 @@ int nv_start_rc_timer(
     return 0;
 }
 
-int nv_stop_rc_timer(
+int NV_API_CALL nv_stop_rc_timer(
     nv_state_t *nv
 )
 {
@@ -1376,7 +1539,14 @@ int nv_stop_rc_timer(
     return 0;
 }
 
-void* nv_get_adapter_state(
+void NV_API_CALL nv_set_dma_address_size(
+    nv_state_t *nv,
+    U032 phys_addr_bits
+)
+{
+}
+
+void* NV_API_CALL nv_get_adapter_state(
     U016 bus,
     U016 slot
 )
@@ -1403,8 +1573,15 @@ void* nv_get_adapter_state(
     return NULL;
 }
 
-void nv_verify_pci_config(nv_state_t *nv, BOOL do_the_bars)
+void NV_API_CALL nv_verify_pci_config(
+    nv_state_t *nv,
+    BOOL check_the_bars
+)
 {
-    NV_PCI_CHECK_CONFIG_SPACE(nv, do_the_bars);
-}
+    nv_stack_t *sp;
+    struct nvidia_softc *sc = nv->os_state;
 
+    sp = sc->pci_cfgchk_sp;
+
+    NV_PCI_CHECK_CONFIG_SPACE(sp, nv, check_the_bars, FALSE, FALSE);
+}
