@@ -30,8 +30,6 @@ void nvidia_sysctl_init(void)
 {
     device_t dev;
     struct sysctl_oid *oid;
-
-    U032 i = 0;
     nv_parm_t *entry;
 
     sysctl_ctx_init(&sysctl_ctx);
@@ -100,18 +98,17 @@ void nvidia_sysctl_init(void)
             0,
             "NVIDIA SYSCTL Registry Node");
 
+    entry = nv_parms;
     do {
-        entry = &nv_parms[i++];
-        if (entry->valid) {
-            SYSCTL_ADD_PROC(&sysctl_ctx,
-                    SYSCTL_CHILDREN(oid_registry),
-                    OID_AUTO,
-                    entry->name,
-                    CTLTYPE_UINT | CTLFLAG_RW | CTLFLAG_DYN,
-                    entry->data, 0,
-                    nvidia_sysctl_registry_key,
-                    "IU", NULL);
-        }
+        SYSCTL_ADD_PROC(&sysctl_ctx,
+            SYSCTL_CHILDREN(oid_registry),
+            OID_AUTO,
+            entry->name,
+            CTLTYPE_UINT | CTLFLAG_RW | CTLFLAG_DYN,
+            entry->data, 0,
+            nvidia_sysctl_registry_key,
+            "IU", NULL);
+        entry++;
     } while(entry->name != NULL);
 
     option_string = malloc(1, M_NVIDIA, M_WAITOK | M_ZERO);
@@ -268,26 +265,38 @@ static void* nvidia_find_bridge(void)
 int nvidia_sysctl_dev_model(SYSCTL_HANDLER_ARGS)
 {
     nv_state_t *nv = arg1;
+    nv_stack_t *sp;
 
     char model_name[NV_DEVICE_NAME_LENGTH+1];
     U016 id = nv->device_id;
 
-    if (rm_get_device_name(NULL, nv, id, NV_DEVICE_NAME_LENGTH, model_name)
+    NV_UMA_ZONE_ALLOC_STACK(sp);
+    if (sp == NULL)
+        return ENOMEM;
+
+    if (rm_get_device_name(sp, nv, id, NV_DEVICE_NAME_LENGTH, model_name)
             != RM_OK) {
         strcpy(model_name, "Unknown");
     }
 
+    NV_UMA_ZONE_FREE_STACK(sp);
+
     return SYSCTL_OUT(req, model_name, strlen(model_name) + 1);
 }
 
 int nvidia_sysctl_dev_vbios(SYSCTL_HANDLER_ARGS)
 {
     nv_state_t *nv = arg1;
+    nv_stack_t *sp;
 
     U032 vbios[5];
     U008 vbios_version[16];
 
-    if (rm_get_vbios_version(NULL, nv, &vbios[0], &vbios[1], &vbios[2],
+    NV_UMA_ZONE_ALLOC_STACK(sp);
+    if (sp == NULL)
+        return ENOMEM;
+
+    if (rm_get_vbios_version(sp, nv, &vbios[0], &vbios[1], &vbios[2],
                 &vbios[3], &vbios[4]) != RM_OK) {
         /*
          * The VBIOS version is only accessible after the device has been
@@ -299,6 +308,8 @@ int nvidia_sysctl_dev_vbios(SYSCTL_HANDLER_ARGS)
                 vbios[1], vbios[2], vbios[3], vbios[4]);
     }
 
+    NV_UMA_ZONE_FREE_STACK(sp);
+
     return SYSCTL_OUT(req, vbios_version, strlen(vbios_version) + 1);
 }
 
@@ -348,7 +359,7 @@ int nvidia_sysctl_registry_dwords(SYSCTL_HANDLER_ARGS)
 
     new_option_string = malloc((len + 1), M_NVIDIA, M_WAITOK);
     if (!new_option_string)
-        return -ENOMEM;
+        return ENOMEM;
 
     error = SYSCTL_IN(req, new_option_string, len);
     if (error)
