@@ -116,6 +116,8 @@ NvU64       NV_API_CALL  os_get_current_pdpte        (U032);
 U032        NV_API_CALL  os_get_cpu_count            (void);
 RM_STATUS   NV_API_CALL  os_raise_smp_barrier        (void);
 RM_STATUS   NV_API_CALL  os_clear_smp_barrier        (void);
+RM_STATUS   NV_API_CALL  os_disable_console_access   (void);
+RM_STATUS   NV_API_CALL  os_enable_console_access    (void);
 RM_STATUS   NV_API_CALL  os_registry_init            (void);
 
 void*       NV_API_CALL  os_map_kernel_space_high    (U032, U032);
